/*
		Splash.cpp : implementation file

        Copyright (C) 2000 Patrick Dughi

        This software may be used and distributed according to the terms
        of the GNU Public License, incorporated herein by reference, located
		as the file gpl.txt in this same directory.

        The author may be reached at dughi@imaxx.net.
.		
		Custom implementation of 256 color splash screens
*/
#include "stdafx.h"
#include "Cbreak.h"
#include "Splash.h"
#include "utils.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSplash

CSplash::CSplash(UINT nBitmapID, UINT nDuration /*= 2500= */)
{
	m_nBitmapID = nBitmapID;
	m_nDuration = nDuration;
}

BEGIN_MESSAGE_MAP(CSplash, CWnd)
	//{{AFX_MSG_MAP(CSplash)
	ON_WM_PAINT()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



BOOL CSplash::Create()
{
	HDC hdc = ::CreateDC("DISPLAY",NULL,NULL,NULL); 
    CDC screenDC;
    int cx,cy;
    screenDC.Attach(hdc); 
	
	cx = screenDC.GetDeviceCaps(HORZRES);
	cy = screenDC.GetDeviceCaps(VERTRES);

	if( !GetBitmapAndPalette(m_nBitmapID, m_bitmap, m_pal) )
	{
		TRACE1( "Could not load bitmap resource - %d\n", m_nBitmapID );
		return FALSE;
	}


	BITMAP bm;
	m_bitmap.GetObject(sizeof(BITMAP), &bm);
	
	// First create an invisible window
	m_wndInvisible.CreateEx(WS_EX_TOPMOST, 
			AfxRegisterWndClass(CS_CLASSDC), 
			_T(""), WS_POPUP, 0, 0, 
			(cx * 4)/5 , (cy * 4)/5, NULL, NULL);

	// Create the the splash window with invisible parent as parent
	BOOL bRetVal = CWnd::CreateEx(WS_EX_TOPMOST, 
			AfxRegisterWndClass(CS_CLASSDC), 
			_T(""), WS_POPUP, 0, 0, 
		//	bm.bmWidth, bm.bmHeight, m_wndInvisible.m_hWnd, NULL);
			(cx * 4)/5, (cy * 4)/5,m_wndInvisible.m_hWnd, NULL);

	CenterWindow();
	ShowWindow(SW_SHOW);
	UpdateWindow();
	
	//Create the timer.
	m_nTimerID = SetTimer(1, m_nDuration, NULL);
	ASSERT(m_nTimerID);
	
	return bRetVal;
}


BOOL CSplash::GetBitmapAndPalette(UINT nIDResource, CBitmap &bitmap, CPalette &pal)
{
	LPCTSTR lpszResourceName = (LPCTSTR)nIDResource;

	HBITMAP hBmp = (HBITMAP)::LoadImage( AfxGetInstanceHandle(), 
			lpszResourceName, IMAGE_BITMAP, 0,0, LR_CREATEDIBSECTION );

	if( hBmp == NULL ) 
		return FALSE;

	bitmap.Attach( hBmp );

	// Create a logical palette for the bitmap
	DIBSECTION ds;
	BITMAPINFOHEADER &bmInfo = ds.dsBmih;
	bitmap.GetObject( sizeof(ds), &ds );

	int nColors = bmInfo.biClrUsed ? bmInfo.biClrUsed : 1 << bmInfo.biBitCount;

	// Create a halftone palette if colors > 256. 
	CClientDC dc(NULL);			// Desktop DC
	if( nColors > 256 )
		pal.CreateHalftonePalette( &dc );
	else
	{
		// Create the palette

		RGBQUAD *pRGB = new RGBQUAD[nColors];
		CDC memDC;
		memDC.CreateCompatibleDC(&dc);

		memDC.SelectObject( &bitmap );
		::GetDIBColorTable( memDC, 0, nColors, pRGB );

		UINT nSize = sizeof(LOGPALETTE) + (sizeof(PALETTEENTRY) * nColors);
		LOGPALETTE *pLP = (LOGPALETTE *) new BYTE[nSize];

		pLP->palVersion = 0x300;
		pLP->palNumEntries = nColors;

		for( int i=0; i < nColors; i++)
		{
			pLP->palPalEntry[i].peRed = pRGB[i].rgbRed;
			pLP->palPalEntry[i].peGreen = pRGB[i].rgbGreen;
			pLP->palPalEntry[i].peBlue = pRGB[i].rgbBlue;
			pLP->palPalEntry[i].peFlags = 0;
		}

		pal.CreatePalette( pLP );

		delete[] pLP;
		delete[] pRGB;
	}

	return TRUE;
}

void CSplash::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	HDC hdc = ::CreateDC("DISPLAY",NULL,NULL,NULL); 
    CDC screenDC;
    int cx,cy;
    screenDC.Attach(hdc); 
	
	cx = screenDC.GetDeviceCaps(HORZRES);
	cy = screenDC.GetDeviceCaps(VERTRES);
	BITMAP bm;
	m_bitmap.GetObject(sizeof(BITMAP), &bm);

	// Create a memory DC compatible with the paint DC
	CDC memDC;
/*	memDC.CreateCompatibleDC( &dc );

	CBitmap *pBmpOld = memDC.SelectObject( &m_bitmap );

	// Select and realize the palette
	if( dc.GetDeviceCaps(RASTERCAPS) & RC_PALETTE && m_pal.m_hObject != NULL )
	{
		dc.SelectPalette( &m_pal, FALSE );
		dc.RealizePalette();
	}

*/
	memDC.CreateCompatibleDC( &screenDC );

	CBitmap *pBmpOld = memDC.SelectObject( &m_bitmap );

	// Select and realize the palette
	if( screenDC.GetDeviceCaps(RASTERCAPS) & RC_PALETTE && m_pal.m_hObject != NULL )
	{
		screenDC.SelectPalette( &m_pal, FALSE );
		screenDC.RealizePalette();
	}


	// Window is same size as bitmap
	CRect rcWnd;
	GetWindowRect( &rcWnd );
	screenDC.SetStretchBltMode(HALFTONE);

	screenDC.StretchBlt((cx/10)-13, (cy/10)-13, rcWnd.Width(),rcWnd.Height(), &memDC, 0, 0, bm.bmWidth,
		bm.bmHeight, SRCCOPY);
 //   dc.BitBlt(0,0, rcWnd.Width(), rcWnd.Height(), &memDC, 0, 0, SRCCOPY);
	// Restore bitmap in memDC
	memDC.SelectObject( pBmpOld );
	
	// Do not call CWnd::OnPaint() for painting messages
}

void CSplash::OnTimer(UINT nIDEvent) 
{
	if (m_nTimerID == nIDEvent)
	{	
		//Destroy the timer and splash window
		KillTimer(m_nTimerID);
		m_wndInvisible.DestroyWindow(); 	
		delete this;
		return;
	}	 
	
	CWnd::OnTimer(nIDEvent);
}

BOOL CSplash::PreTranslateMessage(MSG* pMsg) 
{
	ASSERT(pMsg != NULL);
	
	if (pMsg->message == WM_KEYDOWN ||
			pMsg->message == WM_SYSKEYDOWN ||
			pMsg->message == WM_LBUTTONDOWN ||
			pMsg->message == WM_RBUTTONDOWN ||
			pMsg->message == WM_MBUTTONDOWN )
	{
		//Destroy the timer and splash window
		KillTimer(m_nTimerID);
		m_wndInvisible.DestroyWindow(); 	
		delete this;
		return 1;
	}	 
	
	return CWnd::PreTranslateMessage(pMsg);
}
