/*
		circleFile.cpp: implementation of the CircleFile class.

        Copyright (C) 2000 Patrick Dughi

        This software may be used and distributed according to the terms
        of the GNU Public License, incorporated herein by reference, located
		as the file gpl.txt in this same directory.

        The author may be reached at dughi@imaxx.net.

        This class contains file access classes, specifically for use in reading a circlemud file.
	
*/

#pragma warning(disable:4786) 

#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include "stdafx.h"
#include "Cbreak.h"
#include "circleFile.h"
#include "utils.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CircleFile::CircleFile()
{
  
}

CircleFile::~CircleFile()
{

}

bool CircleFile::open_file(string fname, int file_mode,int zone=0)
{
  string hex=fname;
  while(hex.find("\\\\") != -1) {
	hex.replace(hex.find("\\\\"),2,"\\");
  }
  if(the_file.is_open()) {
    close_file();
  }
  the_file.clear();

  if(file_mode == 0 ) {
		the_file.open(hex.c_str(),ios::nocreate | ios::in);
	} else {
    the_file.open(hex.c_str(), ios::out);
	}

	if(!the_file.good()) {
    error("File %s DNE\n",fname.c_str());
	  return false;
  }
  mode = file_mode;
  the_zone=zone;
  return true;
}

int CircleFile::get_line(string *line)
{
	char a[4097];

	line->erase();

	if(!the_file.good()) {
	  return false;
	}
    
  the_file.getline(a, 4096);
  char_count = strlen(a) +1;     /* total chars advanced in the stream..+1 from */
  if(a[strlen(a)-1] == '\r' || a[strlen(a)-1] == '\n') {   
	  a[strlen(a)-1]= '\0';
	} 
	*line=a;
    char_count = strlen(a);
	return true;
}

int CircleFile::tilde_read(string *buffer)
{
  string temp,work;
  int si;

  buffer->erase();
	temp.erase();
  work.erase();

	if(!the_file.good()) {
		return false;
	}

  char_count = 0;
	while(the_file.good()) {

  	get_line(&work);
    char_count = work.size() +1;

		if(work.rfind("~") != -1) {
				/* if it's at the end..good */
			si= work.rfind("~");
      /* +1/2 necessary because documents don't say how to include a ~ in a     */
			/* multiline desc, ie, "this blanket is ~3 feet long~" ... so, i'll assume*/
			/* if we have the 'last' tilde in a group, and it's near the end of the   */
			/* line, followed perhaps only by a single char (stock zone bug) then it's*/
			/* good																														        */

			if(si == work.size() || si+1 == work.size() || si+2 == work.size()) {
				/* we've found our man, now, lets strip the extra \r\n off just in case 
					 if it exists from a previous get_line in this function */
				if(si <= 2 && buffer->size() >= 2 &&
					  buffer->at(buffer->size()-1) == '\n' &&
						buffer->at(buffer->size()-2) == '\r') {
  				buffer->resize(buffer->size() - 2);
				}
				work.resize(si);
				*buffer += work;
				return TRUE;
			} else {
				*buffer += work;
				/* \n automatically removed */
				*buffer += "\r\n";
				continue;
			} 
		} else {
			*buffer +=work;
			*buffer += "\r\n";			
		}
	}
	return FALSE;
}

int CircleFile::close_file()
{
  the_file.close();
  the_file.clear();
  return true;
}

int CircleFile::rewind_file()
{
	/* mode 0 = read, mode 1 = write */
  if(mode == 0) {
    the_file.seekg(0);
  } else {
	the_file.seekp(0);
  }
  return true;
}

bool CircleFile::write_line(string *line)
{
  the_file << line->c_str();
  return true;
}

int CircleFile::eof()
{
  return (the_file.eof());
}

void CircleFile::rewind_oneline()
{
	char_count++;
  if(mode == 0) {
		the_file.seekg(the_file.tellg() - (long)char_count);
	} else {
		the_file.seekp(the_file.tellp() - (long)char_count);
	}
	char_count=0;
}

bool CircleFile::write_line(string line)
{
  return(write_line(&line));
}

bool CircleFile::file_exists(string *fname)
{
  if(the_file.is_open()) {
    return TRUE;
  }
  the_file.clear();

  the_file.open(fname->c_str(),ios::nocreate | ios::in);
  if(!the_file.good()) {
    return false;
  }
  the_file.close();
  the_file.clear();
  return TRUE;
}
